/*
 * Copyright (c) 1999, 2013, Oracle and/or its affiliates. All rights reserved.
 * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

import java.security.cert.*;
import java.security.*;
import java.util.*;
import java.io.*;

/**
 * Certificate chain validation test
 
 * @author Artem Smotrakov
 */
public class CertValidationTest {

    private List<X509Certificate> certs = new ArrayList<>();
    private X509Certificate trustedCert;
    private String ocspResponder = "";

    public static void main(String[] args) throws Exception {
        CertValidationTest test = new CertValidationTest();
        test.runTest(args);
    }

    /**
     * Parse command line parameters
     */
    private void parseArgs(String[] args) throws Exception {
	int i = 0;
        String arg;
        while (i < args.length && args[i].startsWith("-")) {
            arg = args[i++];

            if (arg.equals("-ocsp_responder")) {
                if (i < args.length) {
            	    ocspResponder = args[i++];
                } else {
                    System.err.println("-ocsp_responder requires OCSP responder URL");
                }
            }
            else if (arg.equals("-trusted_cert")) {
                if (i < args.length) {
            	    trustedCert = loadCertificate(args[i++]);
                } else {
                    System.err.println("-trusted_cert requires a path to trusted certificate");
                }
            }
            else if(arg.equals("-certpath")) {
        	if (i < args.length) {
        	    while (i < args.length && !args[i].startsWith("-")) {
        		certs.add(loadCertificate(args[i++]));
        	    }
        	} else {
        	    System.err.println("-certpath requires paths to certificates");
        	}
            }
        }
    }

    /**
     * @param args Command line parameters
     */
    private void runTest(String[] args) throws Exception {
        parseArgs(args);
        doValidation();
    }

    /**
     * Run certificate validation.
     */
    private void doValidation() throws CertificateException, NoSuchAlgorithmException, InvalidAlgorithmParameterException, CertPathValidatorException {
        Security.setProperty("ocsp.responderURL", ocspResponder);
        Security.setProperty("ocsp.enable", "true");
        
        Set<TrustAnchor> trustedCertsSet = new HashSet<>();
        trustedCertsSet.add(new TrustAnchor(trustedCert, null));

        CertificateFactory cf = CertificateFactory.getInstance("X509");
        CertPath cp = (CertPath) cf.generateCertPath(certs);

        CertPathValidator cpv = CertPathValidator.getInstance("PKIX");
        
        PKIXParameters params = new PKIXParameters(trustedCertsSet);        
        cpv.validate(cp, params);

        System.out.println("Validation passed");
    }

    /**
     * Load a X509 certificate from file.
     */
    public static X509Certificate loadCertificate(String path) throws IOException, CertificateException {
        FileInputStream fis = new FileInputStream(path);
        CertificateFactory cf = CertificateFactory.getInstance("X509");
        return (X509Certificate) cf.generateCertificate(fis);
    }

}
